@@ -1,6 +1,6 @@
 ;;; diredfd.el --- Dired functions and settings to mimic FD/FDclone
 ;;
-;; Copyright (c) 2014-2024 Akinori MUSHA
+;; Copyright (c) 2014-2026 Akinori MUSHA
 ;;
 ;; All rights reserved.
 ;;
@@ -28,8 +28,8 @@
 ;; Author: Akinori MUSHA <knu@iDaemons.org>
 ;; URL: https://github.com/knu/diredfd.el
 ;; Created: 25 Dec 2014
-;; Package-Version: 20241209.623
-;; Package-Revision: d789710c7e96
+;; Package-Version: 20260206.337
+;; Package-Revision: a3eb656d2bd2
 ;; Package-Requires: ((emacs "28.1"))
 ;; Keywords: unix, directories, dired
 
@@ -115,6 +115,15 @@
   :type 'boolean
   :group 'diredfd)
 
+(defcustom diredfd-sort-inhibit-after-readin-functions
+  '(diff-hl-dired-update)
+  "Functions to inhibit in `dired-after-readin-hook' after sorting.
+These functions are skipped when running the hook after sorting,
+which is useful for avoiding issues with VC-related functions that
+expect a fresh buffer read rather than a sort operation."
+  :type '(repeat function)
+  :group 'diredfd)
+
 (defvar diredfd-enter-history nil
   "Default variable for `diredfd-enter-history-variable'.")
 
@@ -175,7 +184,7 @@ Adds a return VALUE to `file-name-history'."
 (defun diredfd--ad-auto-revert-if-sync (&rest args)
   "Internal advice function of diredfd.  ARGS is ignored."
   (or (bound-and-true-p dired-async-be-async)
-                                (diredfd-auto-revert)))
+      (diredfd-auto-revert)))
 
 (defun diredfd--ad-auto-revert-and-restore-point (&rest args)
   "Internal advice function of diredfd.  ARGS is ignored."
@@ -1127,9 +1136,15 @@ SORT-KEY and SORT-DIRECTION are asked in interactive mode."
            diredfd-sort-key
            (if (diredfd-sort-desc-p) "descending" "ascending")))
 
+(defvar diredfd--sorting nil
+  "Non-nil while `diredfd-sort' is running.")
+
 (defun diredfd-sort ()
-  (let* ((current (or (dired-get-filename nil t) (concat (file-name-as-directory default-directory) "..")))
-         (buffer-read-only))
+  "Sort directory entries in the current dired buffer."
+  (unless diredfd--sorting
+    (let* ((diredfd--sorting t)
+           (current (or (dired-get-filename nil t) (concat (file-name-as-directory default-directory) "..")))
+           (buffer-read-only))
       (goto-char (point-min))
       (while (cl-loop while (dired-between-files)
                       do (if (eobp)
@@ -1140,8 +1155,12 @@ SORT-KEY and SORT-DIRECTION are asked in interactive mode."
           (while (not (dired-between-files))
             (forward-line))
           (diredfd-sort-lines beg (point))))
-      (dired-goto-file current))
-  (set-buffer-modified-p nil))
+      (dired-goto-file current)
+      (set-buffer-modified-p nil)
+      (let ((dired-after-readin-hook
+             (cl-set-difference dired-after-readin-hook
+                                diredfd-sort-inhibit-after-readin-functions)))
+        (run-hooks 'dired-after-readin-hook)))))
 
 (defcustom diredfd-highlight-line t
   "If non-nil, the current line is highlighted like FDclone."
@@ -1196,7 +1215,6 @@ SORT-KEY and SORT-DIRECTION are asked in interactive mode."
         (advice-add #'dired-async-after-file-create :around #'diredfd--ad-auto-revert-async))
 
     (setq diredfd--enabled t))
-  (add-hook 'dired-after-readin-hook 'diredfd-dired-after-readin-setup t t)
   (setq-local dired-deletion-confirmer 'y-or-n-p)
   (face-remap-set-base 'dired-directory 'diredfd-directory)
   (face-remap-set-base 'dired-symlink   'diredfd-symlink)
@@ -1228,7 +1246,6 @@ SORT-KEY and SORT-DIRECTION are asked in interactive mode."
         (advice-remove #'dired-async-after-file-create #'diredfd--ad-auto-revert-async))
 
     (setq diredfd--enabled nil))
-  (remove-hook 'dired-after-readin-hook 'diredfd-dired-after-readin-setup t)
   (kill-local-variable 'dired-deletion-confirmer)
   (face-remap-reset-base 'dired-directory)
   (face-remap-reset-base 'dired-symlink)
@@ -1296,12 +1313,14 @@ SORT-KEY and SORT-DIRECTION are asked in interactive mode."
 ;;;###autoload
 (defun diredfd-enable ()
   "Enable diredfd that makes `dired' look and feel like FD/FDclone."
-  (add-hook 'dired-mode-hook 'diredfd-mode))
+  (add-hook 'dired-mode-hook 'diredfd-mode)
+  (add-hook 'dired-after-readin-hook 'diredfd-dired-after-readin-setup))
 
 ;;;###autoload
 (defun diredfd-disable ()
   "Disable diredfd that makes `dired' look and feel like FD/FDclone."
-  (remove-hook 'dired-mode-hook 'diredfd-mode))
+  (remove-hook 'dired-mode-hook 'diredfd-mode)
+  (remove-hook 'dired-after-readin-hook 'diredfd-dired-after-readin-setup))
 
 (provide 'diredfd)
 
